import { useNuxtApp, useRuntimeConfig } from "#imports";
function getEndpoint(operation, operationName) {
  const config = useRuntimeConfig();
  return `${config?.public?.["nuxt-graphql-middleware"]?.serverApiPrefix}/${operation}/${operationName}`;
}
export const useGraphqlState = () => {
  const nuxtApp = useNuxtApp();
  return nuxtApp?._graphql_middleware;
};
export function useGraphqlQuery(...args) {
  const name = args[0];
  if (typeof name !== "string") {
    return Promise.reject(new Error("Invalid query name"));
  }
  const variables = args[1] || {};
  let params = {};
  const queryFallback = Object.keys(variables).some((key) => {
    const valueType = typeof variables[key];
    return valueType === "function" || valueType === "object";
  });
  if (queryFallback) {
    params.__variables = JSON.stringify(variables);
  } else {
    params = variables;
  }
  const state = useGraphqlState();
  return $fetch(getEndpoint("query", name), {
    params,
    ...state.value.fetchOptions
  });
}
export function useGraphqlMutation(...args) {
  const state = useGraphqlState();
  const name = args[0];
  const body = args[1] || {};
  if (typeof name !== "string") {
    return Promise.reject(new Error("Invalid mutation name"));
  }
  return $fetch(getEndpoint("mutation", name), {
    method: "post",
    body,
    ...state.value.fetchOptions
  });
}
