import {
  defineEventHandler,
  createError,
  getQuery,
  getMethod,
  readBody
} from "h3";
import { loadNuxtConfig } from "@nuxt/kit";
import { useRuntimeConfig } from "#imports";
import operations from "#graphql-documents";
var GraphqlMiddlewareOperation = /* @__PURE__ */ ((GraphqlMiddlewareOperation2) => {
  GraphqlMiddlewareOperation2["Query"] = "query";
  GraphqlMiddlewareOperation2["Mutation"] = "mutation";
  return GraphqlMiddlewareOperation2;
})(GraphqlMiddlewareOperation || {});
let moduleConfig = null;
function getModuleConfig() {
  if (moduleConfig) {
    return Promise.resolve(moduleConfig);
  }
  const { graphqlMiddleware } = useRuntimeConfig();
  return loadNuxtConfig({
    cwd: graphqlMiddleware.rootDir
  }).then((v) => {
    moduleConfig = v.graphqlMiddleware;
    return v.graphqlMiddleware;
  });
}
function queryParamToVariables(query) {
  try {
    if (query.__variables && typeof query.__variables === "string") {
      return JSON.parse(query.__variables);
    }
  } catch (_e) {
  }
  return query;
}
function getEndpoint(moduleConfig2, event, operation, operationName) {
  if (typeof moduleConfig2.graphqlEndpoint === "string") {
    return moduleConfig2.graphqlEndpoint;
  }
  return moduleConfig2.graphqlEndpoint(event, operation, operationName);
}
function getFetchOptions(moduleConfig2, event, operation, operationName) {
  if (typeof moduleConfig2.serverFetchOptions === "function") {
    return moduleConfig2.serverFetchOptions(event, operation, operationName) || {};
  } else if (typeof moduleConfig2.serverFetchOptions === "object") {
    return moduleConfig2.serverFetchOptions;
  }
  return {};
}
export default defineEventHandler(async (event) => {
  const method = getMethod(event);
  const operation = event.context.params.operation;
  if (method === "POST" && operation !== "mutation" /* Mutation */) {
    throw createError({
      statusCode: 400,
      statusMessage: "Mutations must be a POST request."
    });
  }
  if (method === "GET" && operation !== "query" /* Query */) {
    throw createError({
      statusCode: 400,
      statusMessage: "Queries must be a GET request."
    });
  }
  const name = event.context.params.name;
  const query = operations[operation][name];
  if (!query) {
    throw createError({
      statusCode: 400,
      statusMessage: `Operation "${operation}" with name "${name} not found."`
    });
  }
  const config = await getModuleConfig();
  const endpoint = getEndpoint(config, event, operation, name);
  const fetchOptions = getFetchOptions(config, event, operation, name);
  const variables = operation === "query" /* Query */ ? queryParamToVariables(getQuery(event)) : readBody(event);
  return $fetch(endpoint, {
    method: "POST",
    body: {
      query,
      variables
    },
    ...fetchOptions
  }).catch((err) => {
    throw createError({
      statusCode: 500,
      statusMessage: "Couldn't execute GraphQL query.",
      data: err && "message" in err ? err.mess : err
    });
  });
});
